@@ -154,7 +154,7 @@ int parse_options(int argc, char *untrusted_argv[], int *input_fds,
                     untrusted_argv[optind - 1]);
             exit(1);
         }
-        if (opt == 'k' || opt == 'K' || opt == opt_export) {
+        if (opt == 'k' || opt == 'K' || opt == opt_export || opt == opt_export_ssh) {
             // --list-keys and --export can have multiple key IDs as arguments
             mode_list_keys = 1;
         } else if (opt == opt_status_fd) {
